package forum;

import exceptions.*;
import util.Mensagem;

import java.util.Vector;
import java.sql.SQLException;




public class CadastroForum implements ICadastroForum {

	
	private IRepositorioForum rep;
	
	
	
	public CadastroForum(IRepositorioForum r)  {
		this.rep = r;
	}
		
		
	
	
	public void adicionarForum(Forum f) throws ObjetoNuloException,SQLException {
		if(f == null) {
			throw new ObjetoNuloException();
		}		
		this.rep.adicionarForum(f);
	}
	
	public void adicionarAssunto(int id_forum, Assunto a) 
	  throws ObjetoNuloException,ForumNaoExistenteException,SQLException {
		if(a == null) {
			throw new ObjetoNuloException();
		}
		if(!this.existeForum(id_forum)) {
			throw new ForumNaoExistenteException();
		}
		this.rep.adicionarAssunto(id_forum,a);
	}
	
	
	public void adicionarTopico(int id_forum, int id_assunto, Topico t) 
	  throws ObjetoNuloException,ForumNaoExistenteException,AssuntoNaoExistenteException,SQLException {
		if(t == null) {
			throw new ObjetoNuloException();
		}
		if(!this.existeForum(id_forum)) {
			throw new ForumNaoExistenteException();
		}
		if(!this.existeAssunto(id_forum,id_assunto)) {
			throw new AssuntoNaoExistenteException();
		}
		this.rep.adicionarTopico(id_forum,id_assunto,t);		
	}
	
	
	public void adicionarMensagemTopico(int id_forum, int id_assunto,int id_topico, Mensagem m) 
	  throws ObjetoNuloException,ForumNaoExistenteException,AssuntoNaoExistenteException,
	  TopicoNaoExistenteException,SQLException {
		if(m == null) {
			throw new ObjetoNuloException();
		}
		
		if(!this.existeForum(id_forum)) {
			throw new ForumNaoExistenteException();
		}
		if(!this.existeAssunto(id_forum,id_assunto)) {
			throw new AssuntoNaoExistenteException();
		}
		if(!this.existeTopico(id_forum,id_assunto,id_topico)) {
			throw new TopicoNaoExistenteException();
		}	
		
		this.rep.adicionarMensagemTopico(id_forum,id_assunto,id_topico,m);
	}
	
		
	
	public boolean existeForum(int id_forum) throws SQLException {
		try {
			this.procurarForum(id_forum);
		}
		catch (ForumNaoExistenteException e) {
			return false;
		}
		return true;
	}
	
	public boolean existeAssunto(int id_forum, int id_assunto) throws SQLException {
		try {
			this.procurarAssunto(id_forum,id_assunto);
		}
		catch (AssuntoNaoExistenteException e) {
			return false;
		}
		return true;
	}
	
	public boolean existeTopico(int id_forum, int id_assunto, int id_topico) throws SQLException {
		try {
			this.procurarTopico(id_forum, id_assunto, id_topico);
		}
		catch (TopicoNaoExistenteException e) {
			return false;
		}
		return true;
	}
	
	public boolean existeMensagemTopico(int id_forum, int id_assunto, int id_topico,int id_mensagem) throws SQLException {
		try {
			this.procurarMensagemTopico(id_forum,id_assunto,id_topico,id_mensagem);
		}
		catch (MensagemTopicoNaoExistenteException e) {
			return false;
		}
		return true;
	}
	
	
	
	
	
	
	public Vector getForuns() throws SQLException {
		return this.rep.getForuns();	
	}	
	
	
	public Vector getTopicosAssunto(int id_forum,int id_assunto) throws SQLException {
		return this.rep.getTopicosAssunto(id_forum,id_assunto);
	}
	
	public Vector getMensagensTopico(int id_forum,int id_assunto,int id_topico) throws SQLException {
		return this.rep.getMensagensTopico(id_forum,id_assunto,id_topico);	
	}
	
	
	
	public Forum procurarForum(int id_forum) throws ForumNaoExistenteException,SQLException {		
		Forum f = this.rep.procurarForum(id_forum);
		if (f == null) {
			throw new ForumNaoExistenteException();
		}
		return f;
	}
	
	public Assunto procurarAssunto(int id_forum,int id_assunto) throws AssuntoNaoExistenteException,SQLException {
		Assunto a = this.rep.procurarAssunto(id_forum,id_assunto);
		if (a == null) {
			throw new AssuntoNaoExistenteException();
		}
		return a;
	}
	
	public Topico procurarTopico(int id_forum,int id_assunto,int id_topico) throws TopicoNaoExistenteException,SQLException {
		Topico t = this.rep.procurarTopico(id_forum,id_assunto,id_topico);
		if (t == null) {
			throw new TopicoNaoExistenteException();
		}
		return t;
	}
		
	public Mensagem procurarMensagemTopico(int id_forum,int id_assunto,int id_topico,int id_mensagem) throws MensagemTopicoNaoExistenteException,SQLException {
		Mensagem m = this.rep.procurarMensagemTopico(id_forum,id_assunto,id_topico,id_mensagem);
		if (m == null) {
			throw new MensagemTopicoNaoExistenteException();
		}
		return m;
	}
	
	
	
	
	public void atualizarForum(Forum f) throws ObjetoNuloException,ForumNaoExistenteException,SQLException {
		if(f == null) {
			throw new ObjetoNuloException();
		}		
		if(!this.existeForum(f.getID())) {
			throw new ForumNaoExistenteException();
		}		
		this.rep.atualizarForum(f);
	}	
	
	public void atualizarAssunto(int id_forum,Assunto a) throws ObjetoNuloException,AssuntoNaoExistenteException,SQLException {
		if(a == null) {
			throw new ObjetoNuloException();
		}		
		if(!this.existeAssunto(id_forum,a.getID())) {
			throw new AssuntoNaoExistenteException();
		}		
		this.rep.atualizarAssunto(id_forum,a);
	}
		
	public void atualizarTopico(int id_forum,int id_assunto,Topico t) throws ObjetoNuloException,TopicoNaoExistenteException,SQLException {
		if(t == null) {
			throw new ObjetoNuloException();
		}		
		if(!this.existeTopico(id_forum,id_assunto,t.getID())) {
			throw new TopicoNaoExistenteException();
		}	
		this.rep.atualizarTopico(id_forum,id_assunto,t);
	}
	
	public void atualizarMensagemTopico(int id_forum,int id_assunto, int id_topico,Mensagem m) 
	  throws ObjetoNuloException,MensagemTopicoNaoExistenteException,SQLException {
		if(m == null) {
			throw new ObjetoNuloException();
		}		
		if(!this.existeMensagemTopico(id_forum,id_assunto,id_topico,m.getID())) {
			throw new MensagemTopicoNaoExistenteException();
		}	
		this.rep.atualizarMensagemTopico(id_forum,id_assunto,id_topico,m);
	}
		
	
	

	public void moverTopico (int id_forum,int id_assunto, int id_topico) 
	  throws AssuntoNaoExistenteException,TopicoNaoExistenteException,SQLException {
		if(!this.existeAssunto( id_forum,id_assunto)) {
			throw new AssuntoNaoExistenteException();
		}	
		if(!this.existeTopico(id_forum,id_assunto,id_topico)) {
			throw new TopicoNaoExistenteException();
		}	
		this.rep.moverTopico(id_assunto,id_topico);		
	}
	
	
	
	
	
	public void removerForum(int id_forum) throws ForumNaoExistenteException,SQLException {
		if(!this.existeForum(id_forum)) {
			throw new ForumNaoExistenteException();
		}		
		
		this.rep.removerForum(id_forum);
	}	

	
	public void removerAssunto(int id_forum,int id_assunto) throws AssuntoNaoExistenteException,SQLException {
		if(!this.existeAssunto(id_forum,id_assunto)) {
			throw new AssuntoNaoExistenteException();
		}
		
		this.rep.removerAssunto(id_forum,id_assunto);
	}
	
	
	public void removerTopico(int id_forum,int id_assunto,int id_topico) throws TopicoNaoExistenteException,SQLException {
		if(!this.existeTopico(id_forum,id_assunto,id_topico)) {
			throw new TopicoNaoExistenteException();
		}
		
		this.rep.removerTopico(id_forum,id_assunto,id_topico);
	}
	
	
	public void removerMensagemTopico(int id_forum,int id_assunto,int id_topico,int id_mensagem) throws MensagemTopicoNaoExistenteException,SQLException {
		if(!this.existeMensagemTopico(id_forum,id_assunto,id_topico,id_mensagem)) {
			throw new MensagemTopicoNaoExistenteException();
		}		
		
		this.removerMensagemTopico(id_forum,id_assunto,id_topico,id_mensagem);
	}

}
