package projeto;


import exceptions.*;
import usuario.Usuario;
import usuario.ICadastroUsuario;
import util.ICadastroMensagem;
import util.Mensagem;

import java.sql.SQLException;
import java.util.Vector;



public class CadastroProjeto  implements ICadastroProjeto { 


	private IRepositorioProjeto repProj;
	private ICadastroMensagem cadMen;
	private ICadastroUsuario cadUsu;
	
	
	
	public CadastroProjeto(IRepositorioProjeto repP,ICadastroMensagem cadM, ICadastroUsuario cadU) {
		this.repProj = repP;	
		this.cadMen = cadM;
		this.cadUsu = cadU;
	}	
	
	
	
	
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/*
	 *  METODOS QUE UTILIZAM O REPOSITORIO DE PROJETOS    
	 */
	
	
	public boolean existeProjeto(int id) throws SQLException {
		try{
			this.procurarProjeto(id);
		} catch(ProjetoNaoExistenteException e) {
			return false;
		}		
		return true;
	}	
	
	
	public Projeto procurarProjeto(int id) throws ProjetoNaoExistenteException,SQLException {
		Projeto p = this.repProj.procurarProjeto(id);
		if(p == null) {
			throw new ProjetoNaoExistenteException();
		}
		return p;
	}
	
	
	public Vector getProjetos() throws SQLException {
		return this.repProj.getProjetos();	 	
	}
		
	public Vector getProjetosAdministrados(int id_usuario) throws UsuarioNaoExistenteException,SQLException {
		Usuario u = this.cadUsu.procurarUsuario(id_usuario);
		if(u == null) {
			throw new UsuarioNaoExistenteException();
		}
		return this.repProj.getProjetosAdministrados(id_usuario);		
	}
	
	public Vector getProjetosMembro(int id_usuario) throws UsuarioNaoExistenteException,SQLException {
		Usuario u = this.cadUsu.procurarUsuario(id_usuario);
		if(u == null) {
			throw new UsuarioNaoExistenteException();
		}
		return this.repProj.getProjetosMembro(id_usuario);		
	}
			
	public Vector getAdministradoresProjeto(int id_projeto) throws ProjetoNaoExistenteException,SQLException {
		if(!this.existeProjeto(id_projeto)) {
			throw new ProjetoNaoExistenteException();
		}
		return this.repProj.getAdministradoresProjeto(id_projeto);
	}	
	
	public Vector getMembrosProjeto(int id_projeto) throws ProjetoNaoExistenteException,SQLException {
		if(!this.existeProjeto(id_projeto)) {
			throw new ProjetoNaoExistenteException();
		}
		return this.repProj.getMembrosProjeto(id_projeto);
	}


	
	
	
	public Projeto adicionarProjeto(Projeto p) 
	  throws ObjetoNuloException,AutorNaoExistenteException,SQLException {
		if (p == null) {
			throw new ObjetoNuloException();
		}
		 
		String login = p.getCriador();
		if(login == null) {
			throw new ObjetoNuloException();
		}		
		if(!this.cadUsu.existeUsuario(login)) {
			throw new AutorNaoExistenteException();
		}		
		
		
		p = this.repProj.adicionarProjeto(p);
		try{
			Usuario u = this.cadUsu.procurarUsuario(p.getCriador());
			this.adicionarParticipante(p.getID(),u.getID());
			this.adicionarAdministrador(p.getID(),u.getID());
		} 
		// Exceoes que nunca irao ocorrer (pelo menos nao deveriam)
		catch(SQLException e){e.printStackTrace();}
		catch(Exception e) { e.printStackTrace();}
		
		return p;
	}	

	
	
	public void atualizarProjeto(Projeto p) 
	  throws ObjetoNuloException,ProjetoNaoExistenteException,SQLException {
		if(p == null) {
			throw new ObjetoNuloException();
		}		
		if(!this.existeProjeto(p.getID())) {
			throw new ProjetoNaoExistenteException();
		}
		
		this.repProj.atualizarProjeto(p);
	}
	
	
	
	public Vector getEsperaProjeto(int id_projeto) throws ProjetoNaoExistenteException,SQLException {
		if(!this.existeProjeto(id_projeto)) {
			throw new ProjetoNaoExistenteException();
		}
		return this.repProj.getEsperaProjeto(id_projeto);
		
	}
	
	public void adicionarEsperaProjeto(int id_projeto, int id_usuario)  
	  throws ProjetoNaoExistenteException,UsuarioNaoExistenteException,SQLException {
		if(!this.existeProjeto(id_projeto)) {
			throw new ProjetoNaoExistenteException();
		}
		Usuario u = this.cadUsu.procurarUsuario(id_usuario);
		if(u == null) {
			throw new UsuarioNaoExistenteException();
		}
		
		this.repProj.adicionarEsperaProjeto(id_projeto,id_usuario);
	}
	
	
	
	public void removerEsperaProjeto(int id_projeto, int id_usuario)
   	  throws ProjetoNaoExistenteException,UsuarioNaoExistenteException,SQLException {
		if(!this.existeProjeto(id_projeto)) {
			throw new ProjetoNaoExistenteException();
		}
		Usuario u = this.cadUsu.procurarUsuario(id_usuario);
		if(u == null) {
			throw new UsuarioNaoExistenteException();
		}
		
		this.repProj.removerEsperaProjeto(id_projeto,id_usuario);
	}
	
	
	
	public void adicionarParticipante(int id_projeto, int id_usuario) 
	  throws ObjetoNuloException,ProjetoNaoExistenteException,UsuarioNaoExistenteException,
	  MembroExistenteException,SQLException {
		if(!this.existeProjeto(id_projeto)) {
			throw new ProjetoNaoExistenteException();
		}		
		Usuario u = this.cadUsu.procurarUsuario(id_usuario);
		if (u == null) {
			throw new UsuarioNaoExistenteException();
		}
		
		Vector membros = this.repProj.getMembrosProjeto(id_projeto);
		if(membros.contains(u.getLogin())) {
			throw new MembroExistenteException();
		}

		this.repProj.adicionarMembroProjeto(id_projeto,id_usuario);
	}
	
	
	
	public void adicionarAdministrador(int id_projeto, int id_usuario) 
	  throws ObjetoNuloException,ProjetoNaoExistenteException,UsuarioNaoExistenteException,
	  AdministradorNaoMembroException,AdministradorExistenteException,SQLException {
		if(!this.existeProjeto(id_projeto)) {
			throw new ProjetoNaoExistenteException();
		}		
		Usuario u = this.cadUsu.procurarUsuario(id_usuario);
		if (u == null) {
			throw new UsuarioNaoExistenteException();
		}
		
		Vector membros = this.repProj.getMembrosProjeto(id_projeto);
		Vector admin = this.repProj.getAdministradoresProjeto(id_projeto);
		if(!membros.contains(u.getLogin())) {
			throw new AdministradorNaoMembroException();
		}
		if(admin.contains(u.getLogin())) {
			throw new AdministradorExistenteException();
		}
		
		this.repProj.adicionarAdminProjeto(id_projeto,id_usuario);
	}
	

	
	public void removerParticipanteProjeto(int id_projeto, int id_usuario) 
	  throws ObjetoNuloException,ProjetoNaoExistenteException,UsuarioNaoExistenteException,
	  MembroNaoExistenteException,SQLException {
		if(!this.existeProjeto(id_projeto)) {
			throw new ProjetoNaoExistenteException();
		}		
		Usuario u = this.cadUsu.procurarUsuario(id_usuario);
		if (u == null) {
			throw new UsuarioNaoExistenteException();
		}
		
		this.repProj.removerMembroProjeto(id_projeto,id_usuario);
	}
	
	
	
	
	public void removerAdministradorProjeto(int id_projeto, int id_usuario) 
	  throws ObjetoNuloException,ProjetoNaoExistenteException,UsuarioNaoExistenteException,
	  AdministradorNaoExistenteException,SQLException {
		if(!this.existeProjeto(id_projeto)) {
			throw new ProjetoNaoExistenteException();
		}		
		Usuario u = this.cadUsu.procurarUsuario(id_usuario);
		if (u == null) {
			throw new UsuarioNaoExistenteException();
		}
		
		this.repProj.removerAdminProjeto(id_projeto,id_usuario);		
	}
	
	
	
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/*
	 *  METODOS QUE UTILIZAM O REPOSITORIO DE MENSAGEM    
	 */
	
		
	
		
	public void adicionarComentarioProjeto(int id_projeto,Mensagem m) 
	  throws ObjetoNuloException,ProjetoNaoExistenteException,MensagemNaoExistenteException,
	  AutorNaoExistenteException,SQLException {
		if(!this.existeProjeto(id_projeto)) {
			throw new ProjetoNaoExistenteException();
		}		
		if(m == null) {
			throw new ObjetoNuloException();
		}
		
		this.cadMen.adicionarComentarioProjeto(id_projeto,m);		
	}
	
	
	public void adicionarRecadoProjeto(int id_projeto, Mensagem m) 
	  throws ObjetoNuloException,ProjetoNaoExistenteException,MensagemNaoExistenteException,
	  AutorNaoExistenteException,SQLException {
		if(!this.existeProjeto(id_projeto)) {
			throw new ProjetoNaoExistenteException();
		}		
		if(m == null) {
			throw new ObjetoNuloException();
		}	
		
		this.cadMen.adicionarRecadoProjeto(id_projeto,m);
	}
	
	
	public void removerComentarioProjeto(int id_projeto, Mensagem m) 
	  throws ObjetoNuloException,ProjetoNaoExistenteException,MensagemNaoExistenteException,
	  ComentarioNaoExistenteException,SQLException {
		if(!this.existeProjeto(id_projeto)) {
			throw new ProjetoNaoExistenteException();
		}
		if(!this.cadMen.existeMensagem(m.getID())) {
			throw new MensagemNaoExistenteException();
		}		
		
		Vector comen = this.cadMen.getComentariosProjeto(id_projeto);
		if(!comen.contains(m)) {
			throw new ComentarioNaoExistenteException();
		}
		
		this.cadMen.removerComentarioProjeto(id_projeto,m.getID());
	}
	
	
	public void removerRecadoProjeto(int id_projeto, Mensagem m) 
	  throws ObjetoNuloException,ProjetoNaoExistenteException,MensagemNaoExistenteException,
	  RecadoNaoExistenteException,SQLException {
		if(!this.existeProjeto(id_projeto)) {
			throw new ProjetoNaoExistenteException();
		}
		if(!this.cadMen.existeMensagem(m.getID())) {
			throw new MensagemNaoExistenteException();
		}		
		
		
		Vector rec = this.cadMen.getRecadosProjeto(id_projeto);
		if(!rec.contains(m)) {
			throw new RecadoNaoExistenteException();
		}
		
		this.cadMen.removerRecadoProjeto(id_projeto,m.getID());
	}	
	
	
	

	public void removerProjeto(int id_projeto) throws ObjetoNuloException,ProjetoNaoExistenteException,SQLException {
		if(!this.existeProjeto(id_projeto)) {
			throw new ProjetoNaoExistenteException();
		}
		
		//////////////////////////////////////////////////////
		/***    ANTES REMOVE TUDO QUE USA ESSE PROJETO    ***/
		Vector temp;
		Mensagem m;		
						
		temp = this.cadMen.getComentariosProjeto(id_projeto);
		while(!temp.isEmpty()) {
			m = (Mensagem)temp.firstElement();
			try {
				this.removerComentarioProjeto(id_projeto,m);
			}
			catch(SQLException e){throw new SQLException();}
			catch(Exception e) {}
		}
		
		temp = this.cadMen.getRecadosProjeto(id_projeto);
		while(!temp.isEmpty()) {
			m = (Mensagem)temp.firstElement();
			try {
				this.removerRecadoProjeto(id_projeto,m);
			}
			catch(SQLException e){throw new SQLException();}
			catch(Exception e) {}
		}		
		//////////////////////////////////////////////////////
		
		
		this.repProj.removerProjeto(id_projeto);
	}	
	
	

}
