package usuario;


import exceptions.*;
import util.Mensagem;
import util.ICadastroMensagem;

import java.sql.SQLException;
import java.util.Vector;



public class CadastroUsuario  implements ICadastroUsuario {
	
	
	private IRepositorioUsuario repositorio;
		
	
	
	public CadastroUsuario(IRepositorioUsuario rep) {
		this.repositorio = rep;		
	}	
	
	
	
	public boolean existeUsuario(String login) throws ObjetoNuloException,SQLException {
		try{
			this.procurarUsuario(login);
		} catch(UsuarioNaoExistenteException e) {
			return false;
		}		
		return true;
	}
	
	
	public Usuario procurarUsuario(String login) 
	  throws ObjetoNuloException,UsuarioNaoExistenteException,SQLException {
		if(login == null) {
			throw new ObjetoNuloException();
		}
		
		Usuario u = this.repositorio.procurarUsuario(login);
		if(u == null) {
			throw new UsuarioNaoExistenteException();
		}
		
		return u;
	}
	
	
	
	public Vector getUsuarios() throws SQLException {
		return this.repositorio.getUsuarios();	 	
	}
	
	public Vector getMensagensPrivada(String login) 
	  throws ObjetoNuloException,UsuarioNaoExistenteException,SQLException {
		if(!this.existeUsuario(login)) {
			throw new UsuarioNaoExistenteException();
		}
		
		return this.repositorio.getMensagensPrivada(login);
	}
	
	
	
	
	public void adicionarUsuario(Usuario u) throws ObjetoNuloException,UsuarioExistenteException,SQLException {
		if (u == null) {
			throw new ObjetoNuloException();
		}
		
		if(this.existeUsuario(u.getLogin())) {
			throw new UsuarioExistenteException();
		}
		
		this.repositorio.adicionarUsuario(u);		
	}	

	
	public void atualizarUsuario(Usuario u) throws ObjetoNuloException,UsuarioNaoExistenteException,SQLException {
		if(u == null) {
			throw new ObjetoNuloException();
		}
		
		if(!this.existeUsuario(u.getLogin())) {
			throw new UsuarioNaoExistenteException();
		}
		
		this.repositorio.atualizarUsuario(u);
	}
	
	
	
	public void adicionarMensagemPrivada(String login, Mensagem m) 
	  throws ObjetoNuloException,UsuarioNaoExistenteException,SQLException {
		if(login == null || m == null) {
			throw new ObjetoNuloException();
		}
		
		Usuario u = this.procurarUsuario(login);
		if(u == null) {
			throw new UsuarioNaoExistenteException();
		}		
		
		this.repositorio.adicionarMensagemPrivada(u.getLogin(),m);
	}
		
	
	
	public void removerMensagemPrivada(int id_mensagem) throws SQLException {				
		this.repositorio.removerMensagemPrivada(id_mensagem);
	}
	
		
	
	public void removerUsuario(String login, ICadastroMensagem cadMen) 
	  throws ObjetoNuloException,UsuarioNaoExistenteException,SQLException {
		if (login == null) {
			throw new ObjetoNuloException();
		}		
		if(!this.existeUsuario(login)) {
			throw new UsuarioNaoExistenteException();
		}
		
		this.repositorio.removerUsuario(login);
		cadMen.deletarAutor(login);			
	}	
	                                                    
	
}