package util;


import exceptions.*;
import usuario.Usuario;
import usuario.CadastroUsuario;

import java.sql.SQLException;
import java.util.Vector;




public class CadastroMensagem implements ICadastroMensagem{

	
	private IRepositorioMensagem repMen;
	private CadastroUsuario cadUsu;
	
	
	public CadastroMensagem(IRepositorioMensagem repMen,CadastroUsuario cadUsu) {
		this.repMen = repMen;
		this.cadUsu = cadUsu;
	}
	
	
	
	public boolean existeMensagem(int id) throws SQLException {
		try{
			this.procurarMensagem(id);
		} catch(MensagemNaoExistenteException e) {
			return false;
		}		
		return true;
	}	
	
	
	public Mensagem procurarMensagem(int id) throws MensagemNaoExistenteException,SQLException {
		Mensagem m = this.repMen.procurarMensagem(id);
		if(m == null) {
			throw new MensagemNaoExistenteException();
		}		
		return m;
	}
	
	
	
	public Vector getMensagensPrivada(int id_usuario) throws SQLException {
		return this.repMen.getMensagensPrivada(id_usuario);
	}
	
	public Vector getComentariosProjeto(int id_projeto) throws SQLException {
		return this.repMen.getComentariosProjeto(id_projeto);
	}
	
	public Vector getRecadosProjeto(int id_projeto) throws SQLException {
		return this.repMen.getRecadosProjeto(id_projeto);
	}
	
	public Vector getMensagensTopico(int id_topico) throws SQLException {
		return this.repMen.getMensagensTopico(id_topico);	
	}
	
	
	
	
	public void atualizarMensagem(Mensagem m) throws ObjetoNuloException,MensagemNaoExistenteException,SQLException {
		if(m == null) {
			throw new ObjetoNuloException();
		}
		if(!this.existeMensagem(m.getID())) {
			throw new MensagemNaoExistenteException();
		}
		
		this.repMen.atualizarMensagem(m);
	}
	
	
	public void deletarAutor(String login) throws ObjetoNuloException,UsuarioNaoExistenteException,SQLException {
		if(login == null) {
			throw new ObjetoNuloException();
		}
		if(!this.cadUsu.existeUsuario(login)) {
			throw new UsuarioNaoExistenteException();
		}
		
		this.repMen.deletarAutorMensagem(login);
	}
	
	
	
		
	public Mensagem adicionarMensagem(Mensagem m)
	  throws ObjetoNuloException,AutorNaoExistenteException,SQLException {
		if(m == null) {
			throw new ObjetoNuloException();
		}
		
		try {
			this.cadUsu.procurarUsuario(m.getAutor());
		}
		catch (UsuarioNaoExistenteException e) {
			throw new AutorNaoExistenteException();
		}
		
		Mensagem temp = this.repMen.adicionarMensagem(m);
		return temp;
	}
	
	
	public Mensagem adicionarMensagemPrivada(int id_usuario, Mensagem m) 
	  throws ObjetoNuloException,AutorNaoExistenteException,SQLException {
		Mensagem temp = this.adicionarMensagem(m);
		this.repMen.adicionarMensagemPrivada(id_usuario,temp.getID());
		
		return temp;
	}
	
	
	public Mensagem adicionarComentarioProjeto(int id_projeto, Mensagem m) 
	  throws ObjetoNuloException,AutorNaoExistenteException,SQLException {
		Mensagem temp = this.adicionarMensagem(m);
		this.repMen.adicionarComentarioProjeto(id_projeto,temp.getID());
		
		return temp;
	}
	
	
	public Mensagem adicionarRecadoProjeto(int id_projeto, Mensagem m) 
	  throws ObjetoNuloException,AutorNaoExistenteException,SQLException {
		Mensagem temp = this.adicionarMensagem(m);
		this.repMen.adicionarRecadoProjeto(id_projeto,temp.getID());
		
		return temp;
	}
	
	
	public void adicionarMensagemTopico(int id_topico, Mensagem m) 
	  throws ObjetoNuloException,AutorNaoExistenteException,SQLException {
		this.repMen.adicionarMensagemTopico(id_topico,m.getID());
	}
	
	
	
	
	public void removerMensagem(int id_mensagem) throws MensagemNaoExistenteException,SQLException {
		if(!this.existeMensagem(id_mensagem)) {
			throw new MensagemNaoExistenteException();
		}
		this.repMen.removerMensagem(id_mensagem);				
	}
	
	
	public void removerMensagemPrivada(int id_usuario, int id_mensagem)
	  throws MensagemNaoExistenteException,SQLException {
		this.removerMensagem(id_mensagem);
		this.repMen.removerMensagemPrivada(id_usuario,id_mensagem);		
	}
	
			
	public void removerComentarioProjeto(int id_projeto, int id_mensagem)
	  throws MensagemNaoExistenteException,SQLException {
		this.removerMensagem(id_mensagem);
		this.repMen.removerComentarioProjeto(id_projeto,id_mensagem);		
	}
	
	
	public void removerRecadoProjeto(int id_projeto, int id_mensagem)
	  throws MensagemNaoExistenteException,SQLException {
		this.removerMensagem(id_mensagem);
		this.repMen.removerRecadoProjeto(id_projeto,id_mensagem);		
	}
	
	
	public void removerMensagemTopico(int id_topico, int id_mensagem)
	  throws MensagemNaoExistenteException,SQLException {
		this.removerMensagem(id_mensagem);
		this.repMen.removerMensagemTopico(id_topico,id_mensagem);		
	}
}
