package util;



import java.io.*;
import java.sql.*;

//import com.mysql.jdbc.Blob;



/**
 * @author cassio
 * <b>Description:</b> Classe que testa algumas das funcionaidades do tipo 
 * de dados BLOB de SQL. 
 */
public class TesteBLOB {

	
	
	/**
	 * Parte da URL para conexao com um SGBD MySQL.
	 */
	static String connectionURL = "jdbc:oracle:thin:@";
		
			
	/**
	 * Metodo usado para inserir um arquivo no banco de dados.
	 * @param co Conexao a ser usada no procedimento.
	 * @param database Nome do banco de dados onde sera inserido 
	 * o arquivo
	 * @param params Atributos do arquivo a ser inserido.
	 *    Sao um total de 5.
	 *    Descricao dos parametros:
	 * 			<i><li>params[0] - nome da musica</li>
	 * 			   <li>params[1] - duracao da musica (segundos)</li>
	 * 			   <li>params[2] - comentarios sobre a musica</li>
	 * 			   <li>params[3] - id do estilo</li>
	 * 			   <li>params[4] - id do usuario</li></i>	  			        
	 * @param arq  Arquivo a ser inserido.
	 * @throws IllegalArgumentException No caso de o numero de
	 * parametros ser diferente de 5
	 * @throws FileNotFoundException Lancada quando o arquivo
	 * nao e encontrado.
	 */
	public static void insertFile(Connection co, String table,
				String[] params,File arq) throws IllegalArgumentException, FileNotFoundException{

		if(params.length != 5) {  //numero de parametros
			throw new IllegalArgumentException("Numero de parametros invalido.");
		}

		String p2 = "(?,?,?,?,?,?,?,?,?)";
		String sql = "insert into " + table + " values" + p2;
		PreparedStatement ps;
		try {
			ps = co.prepareStatement(sql);
			ps.setString(1,arq.getName());
			ps.setString(2,params[0]);
			ps.setInt(3,(int)arq.length());
			ps.setInt(4,Integer.parseInt(params[1]));
			ps.setString(5,params[2]);
			ps.setInt(6,0);
			ps.setInt(7,Integer.parseInt(params[3]));
			ps.setInt(8,Integer.parseInt(params[4]));
			ps.setBinaryStream(9,new FileInputStream(arq),(int)arq.length());
			ps.execute();
			System.out.println("Arquivo inserido com sucesso!!");
		} catch (SQLException e) {
			System.err.println(e.getLocalizedMessage());
			e.printStackTrace();
		}
		
	}	
	
	
	public static void insertImage(Connection co,File img) throws IllegalArgumentException, FileNotFoundException{



	String p2 = "(id_imagem_seq.NEXTVAL,?,?,?)";
	String sql = "insert into imagem values" + p2;
	PreparedStatement ps;
	try {
	
		ps = co.prepareStatement(sql);
		ps.setString(1, img.getName());
		ps.setLong(2, img.length());
		ps.setBinaryStream(3,new FileInputStream(img),(int)img.length());
		ps.execute();
		System.out.println("Arquivo inserido com sucesso!!");
	} catch (SQLException e) {
		System.err.println(e.getLocalizedMessage());
		e.printStackTrace();
	}
	
}
	
	
	/**
	 * @param co Conexao usada na busca
	 * @param fileName Nome do arquivo a ser buscado no banco de dados
	 * @return <li>-2 caso o arquivo nao esteja no banco de dados;</li>
	 * 		   <li>ID do arquivo, se ele estiver no banco de dados</li>
	 */
	public static int isInDatabase(Connection co,String fileName) {
		String sql = "SELECT id FROM teste.arquivo WHERE arquivo.nome=?";
		try {
			PreparedStatement ps = co.prepareStatement(sql);
			ps.setString(1,fileName);
			ResultSet rs = ps.executeQuery();
			if(rs.first()) {
			  	return rs.getInt("id");			  	
			}
		} catch (SQLException e) {			
			e.printStackTrace();
		}
		return -2;
	}
	
	
	
	/**
	 * @param co Conexao com o banco de dados
	 * @param fileName Nome do arquivo a ser salvo do banco de dados
	 * <i><br><b>OBS:</b> O arquivo sera criado no diretorio temporario do usuario, no subdiretorio PAD</i>
	 * @param id ID do arquivo a ser salvo.
	 * @throws EmptyResultSetException Caso o arquivo nao esteja no banco de dados
	 */
	public static void saveTempFile(Connection co,String fileName,int id) throws NoDataException {
		File arq2 = new File(System.getProperty("java.io.tmpdir")+"\\"+System.getProperty("user.name")+"\\PAD");
		
		try {		 
			
			if(!arq2.exists()) {
			  arq2.mkdir();	
			}
			File arq = new File(arq2.getAbsolutePath()+"\\"+fileName);
			FileOutputStream fos = new FileOutputStream(arq);
			String sql = "SELECT file FROM arquivo WHERE id=?";
			PreparedStatement ps = co.prepareStatement(sql);	
			ps.setInt(1,id);
			ResultSet rs = ps.executeQuery();
			if(rs.first()) {
				Blob b = rs.getBlob("file");
				int length = (int)b.length();				
				InputStream fs = (b.getBinaryStream());
			 
				byte mb[] = b.getBytes(1,(int)b.length());
				byte br;
				for(int abcdef = 0; abcdef < mb.length; abcdef++) {
					if((br=(byte)fs.read())!=-1)  	{
						mb[abcdef] = br;	
					}
				}
				fos.write(mb);
			} else {
			    throw new NoDataException(); 	
			}
		} catch (FileNotFoundException e) {			
			e.printStackTrace();
		} catch (SQLException e) {		
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	
	public static void retrieveImage(Connection co,int id) throws NoDataException {
		//File arq2 = new File(System.getProperty("java.io.tmpdir")+"\\"+System.getProperty("user.name")+"\\PAD");
		
		File arq2 = new File("c:\\RECUPEROU");
		
		try {		 
			
			if(!arq2.exists()) {
			  arq2.mkdir();	
			}
			
			///////////////
			Statement stmt = co.createStatement();
            
			ResultSet rset = stmt.executeQuery("SELECT nome from imagem where id='"+id+"'");
			String fileName = "";
			if( rset.next()) {
				fileName = rset.getString(1);
			}
			System.out.println(fileName);
            stmt.close();
			
			
			
				
			///////////////////
			File arq = new File(arq2.getAbsolutePath()+"\\"+fileName);
			FileOutputStream fos = new FileOutputStream(arq);
			String sql = "SELECT text FROM imagem WHERE id=?";
			PreparedStatement ps = co.prepareStatement(sql);	
			ps.setInt(1,id);
			ResultSet rs = ps.executeQuery();
	
			if(rs.next()) {
				Blob b = rs.getBlob("text");
				int length = (int)b.length();				
				InputStream fs = (b.getBinaryStream());
			 
				byte mb[] = b.getBytes(1,(int)b.length());
				byte br;
				for(int abcdef = 0; abcdef < mb.length; abcdef++) {
					if((br=(byte)fs.read())!=-1)  	{
						mb[abcdef] = br;	
					}
				}
				fos.write(mb);
			} else {
			    throw new NoDataException(); 	
			}
		} catch (FileNotFoundException e) {			
			e.printStackTrace();
		} catch (SQLException e) {		
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	
	
	/**
	 * Metodo usado para criar uma conexao com um SGBD MySQL. 
	 * @param host Host onde esta hospedado o SGBD MySQL.
	 * @param port Porta onde o MySQL aceitara conexoes.
	 * @param database Nome do banco de dados.
	 * @param username Nome de usuario usado na conexao.
	 * @param password Senha usada na conexao.
	 * @return Conexao com o banco de dados.
	 * @throws SQLException Caso nao seja possivel a criacao da conexao.
	 */
	public static Connection getOracleJDBCConnection(String host,
				String port,String database,
				String username,String password) 
				throws SQLException {
		//DriverManager.registerDriver(new Driver());
		connectionURL += host+":"+port+":"+database;
		return DriverManager.getConnection(connectionURL,
										  username,password);
		
	}
	
	/**
	 * Metodo usado para criar uma conexao com um SGBD MySQL
	 * @param connectionParams - "URL" com todos os parametros 
	 * necessarios para criar uma conexao com o SGBD.
	 * <br>Exemplo: 
	 * <i>Connection con = Blb.getConnection("jdbc:mysql://HOST:PORT/DATABASE?user=USER&password=PASSWORD");</i>
	 * <br>Onde: <br><b>HOST</b> = host do servidor mysql
	 * <br>		 <b>PORT</b> = porta onde o servidor esta aceitando conexoes
	 * <br>		 <b>DATABASE</b> = banco de dados que sera usado
	 * <br>	     <b>USER</b> = nome de usuario do banco de dados
	 * <br>      <b>PASSWORD</b> = senha para acesso ao banco de dados
	 * @return Conexao com o banco de dados.
	 * @throws SQLException Em caso de erro na conexao.
	 */
	public static Connection getOracleJDBCConnection(String connectionParams) 
				throws SQLException{
		//DriverManager.registerDriver(new Driver());
		return DriverManager.getConnection(connectionParams);
	}
	
	/**
	 * Construtor da classe Blb.
	 */
	public TesteBLOB()  {/*(
		File file = new File("c:/devware.JPG");
		String filename = file.getName();
		try	{
			FileInputStream filin = new FileInputStream(file);
			DriverManager.registerDriver(new com.mysql.jdbc.Driver());
			java.sql.Connection con = (com.mysql.jdbc.Connection)DriverManager.getConnection("jdbc:mysql://localhost:3306/teste","root","");
			String query = "insert into filelib values(?,?,?)";
			java.sql.PreparedStatement pstmt = (PreparedStatement)con.prepareStatement(query);
			pstmt.setInt(1,0);
			pstmt.setString(2,filename);
			pstmt.setBinaryStream(3,(InputStream) filin,(int) file.length());
			pstmt.execute();
			System.out.println("ok!!" );
			
			java.sql.ResultSet rs2;
			long i, f;
			System.out.println(i=System.currentTimeMillis());
			rs2 = pstmt.executeQuery("SELECT * FROM filelib ");
			System.out.println(f=System.currentTimeMillis());
			System.out.println("Tempo de execucao do select: "+(double)(f-i)/1000);
			if(rs2.first()) {
				System.out.println("OK");	
			} else {
				System.out.println("ERRO");	
			}
				
			i=System.currentTimeMillis();
			java.sql.Blob b = rs2.getBlob(3);
			f=System.currentTimeMillis();
			System.out.println("Tempo de execucao do getBlob: " + (double)(f-i)/1000);
		 
			int length = (int)b.length();
			System.out.println(length);
			InputStream fs = (b.getBinaryStream());
		 
			byte mb[] = b.getBytes(1,(int)b.length());
			byte br;
			for(int abcdef = 0; abcdef < mb.length; abcdef++) {
				if((br=(byte)fs.read())!=-1)  	{
					mb[abcdef] = br;	
				}
			}
		
			FileOutputStream fo = new FileOutputStream("c:/temp/mta/teste.mp3");
			i = System.currentTimeMillis();
			fo.write(mb);
			f = System.currentTimeMillis();
			System.out.println("Tempo gasto para salvar o arquivo: " + (double)(f-i)/1000);
			System.out.println(System.getProperty("java.io.tmpdir")+System.getProperty("user.name")+"\\PAD\\musics");
		 
		  }	 catch( SQLException e ) {
		  		e.printStackTrace();
		  } catch( Exception e ) {
		  		e.printStackTrace();
		  }	*/

	}

	/**
	 * Metodo main.
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {
		
		Class.forName("oracle.jdbc.driver.OracleDriver");
        
        Connection con = DriverManager.getConnection
        ("jdbc:oracle:thin:@oracle.cin.ufpe.br:1521:dbgrad", "if68520509", "sixteam2006");
        
        
        System.out.println("CONECTOU");
		
		File f = new File("c:/temp/angelina.jpg");
	
		
		TesteBLOB.insertImage(con,f);
	     System.out.println("GRAVOU");
		
		//TesteBLOB.retrieveImage(con,1);
		//System.out.println("RECUPEROU");
		
		
		
	
		
	}



}

class NoDataException extends Exception {
	NoDataException() {
		super("The data solicited isnt in database.");
	}
}


